<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 20/04/2016
 * Time: 14:43
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Service\Calendar\Restriction\Strategy\RestrictionFactory;
use Util\Messages;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as EntitiesUtil;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\Framework\Log\Exception;


class Meeting
{


    protected $entityManager;
    protected $environment;
    /** @var Task  */
    protected $taskService;
    protected $taskPrecursor;
    protected $resourceProjectService;
    protected $allocationService;
    protected $calendarService;
    /** @var \Factory\Project */
    protected $projectFactory;
    protected $httpKernel;
    protected $repositoryService;
    /** @var Repositories\Project */
    protected $projectRepository;

    /** all CONTROL of PROJECT TYPE */
    const PROJECT_TYPE_ISSUE = Entities\Project::PROJECT_TYPE_ISSUE;
    const PROJECT_TYPE_PRODUCT = Entities\Project::PROJECT_TYPE_PRODUCT;
    const PROJECT_TYPE_STANDARD = Entities\Project::PROJECT_TYPE_STANDARD;
    const PROJECT_TYPE_RELEASE = Entities\Project::PROJECT_TYPE_RELEASE;
    const PROJECT_TYPE_SUB_RELEASE = Entities\Project::PROJECT_TYPE_SUB_RELEASE;
    const PROJECT_TYPE_ERROR = Entities\Project::PROJECT_TYPE_ERROR;


    const PROJECT_CONCLUDED_STATUS = Entities\Project::PROJECT_CONCLUDED_STATUS;
    const PROJECT_PAUSED_STATUS = Entities\Project::PROJECT_PAUSED_STATUS;
    const PROJECT_IN_PROGRESS_STATUS = Entities\Project::PROJECT_IN_PROGRESS_STATUS;
    const PROJECT_NOT_INITIALIZED_STATUS = Entities\Project::PROJECT_NOT_INITIALIZED_STATUS;

    /** @var Repositories\Task */
    protected $taskRepository;

    /** @var Repositories\Structure */
    protected $structureRepository;

    /** @var Repositories\ProjectType */
    protected $projectTypeRepository;

    /** @var Repositories\Resource */
    protected $resourceRepository;

    /** @var Repositories\TaskStatus */
    protected $taskStatusRepository;

    /** @var Repositories\CalendarDates */
    protected $calendarDatesRepository;

    /** @var Repositories\ResourceProject */
    protected $resourceProjectRepository;

    /** @var Repositories\Meeting */
    protected $meetingRepository;

    /** @var Repositories\MeetingProject */
    protected $meetingProjectRepository;

    /** @var \Factory\MeetingProject */
    protected $meetingProjectFactory;
    /** @var \Factory\MeetingTaskStatus */
    protected $meetingTaskStatusFactory;
    /** @var Repositories\Allocation */
    protected $allocationRepository;
    /** @var Repositories\Classification  */
    protected $classificationRepository;
    /** @var Repositories\TaskRelationship  */
    protected $taskRelationshipRepository;
    /** @var Repositories\Attachment */
    protected $attachmentRepository;
    /** @var Repositories\TaskLabel  */
    protected $taskLabelRepository;
    /** @var Repositories\Label */
    protected $labelRepository;
    /** @var Repositories\Comments */
    protected $commentsRepository;
    /**
     * @var \Service\Resource
     */
    private $resourceService;
    /**
     * @var CalendarDetail
     */
    protected $calendarDetailService;
    /**
     * @var \Factory\Meeting
     */
    private $meetingFactory;
    /**
     * @var Project
     */
    private $projectService;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, Task $taskService,
                                ResourceProject $resourceProjectService, TaskPrecursor $taskPrecursor,
                                Allocation $allocationService, Calendar $calendarService, \Factory\Project $projectFactory,
                                Resource $resourceService, \Factory\ResourceProject $resourceProjectFactory,
                                CalendarDetail $calendarDetailService, \Factory\Meeting $meetingFactory,
                                \Factory\MeetingProject $meetingProjectFactory,
                                \Factory\MeetingTaskStatus $meetingTaskStatusFactory, Project $projectService,$directoryBasic)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->taskService = $taskService;
        $this->taskPrecursor = $taskPrecursor;
        $this->resourceProjectService = $resourceProjectService;
        $this->allocationService = $allocationService;
        $this->calendarService = $calendarService;
        $this->projectFactory = $projectFactory;
        $this->resourceService = $resourceService;
        $this->projectRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Project');
        $this->taskRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'task');
        $this->structureRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Structure');
        $this->projectTypeRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'ProjectType');
        $this->resourceRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        $this->taskStatusRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskStatus');
        $this->calendarDatesRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'CalendarDates');
        $this->resourceProjectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ResourceProject');
        $this->meetingRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Meeting');
        $this->meetingProjectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'MeetingProject');
        $this->meetingTaskStatusRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'MeetingTaskStatus');
        $this->allocationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Allocation');
        $this->classificationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Classification');
        $this->taskRelationshipRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskRelationship');
        $this->attachmentRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Attachment');
        $this->taskLabelRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskLabel');
        $this->labelRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Label');
        $this->commentsRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Comments');
        $this->resourceProjectFactory = $resourceProjectFactory;
        $this->calendarDetailService = $calendarDetailService;
        $this->meetingFactory = $meetingFactory;
        $this->meetingProjectFactory = $meetingProjectFactory;
        $this->meetingTaskStatusFactory = $meetingTaskStatusFactory;
        $this->directoryBasicUpload = $directoryBasic . DIRECTORY_SEPARATOR . 'attachments';

        $this->projectService = $projectService;
    }

    public static function getAllTaskStatusControlsISNotConcluded(){
        return array(
            Entities\TaskStatus::TASK_STATUS_CONTROL_NOT_INITIALIZED,
            Entities\TaskStatus::TASK_STATUS_CONTROL_IN_DEVELOPING,
            Entities\TaskStatus::TASK_STATUS_CONTROL_PAUSED,
        );
    }

    public function validateCreateRecurrenceMeetingPlanning($id,\DateTime$beginDate, $beginTime){
        /** @var Entities\Project $meeting */
        $beginDateString = $beginDate->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $meetingPlanning = $this->projectRepository->findCreateRecurrenceMeetingPlanning($id, $beginDateString, $beginTime);
        return $meetingPlanning;
    }

    public function createRecurrenceMeetingPlanning($name, $recurrenceType, $id, $status, $beginDate, $occurrences, $exceptionQuantitative,
                                                    $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                                                    $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay,
                                                    $yearDay, $workTime, $beginTime)
    {
        $meetingPlannings = $this->validateCreateRecurrenceMeetingPlanning($id,$beginDate, $beginTime);
        if(!empty($meetingPlannings)){
            $messagem = '';
            foreach ($meetingPlannings as $meetingPlanning) {
                $messagem .='<BR>'.$meetingPlanning['BEGIN_DATE'];
            }

            if(count($meetingPlannings) >1){
                throw new \Exception("Não foi possível criar a(s) ocorrência(s) de reunião. Ocorrências anteriores (abaixo) não foram finalizadas!<BR><u>$messagem</u>",33);
            }else{
                throw new \Exception("Não foi possível criar a(s) ocorrência(s) de reunião. Ocorrência anterior (abaixo) não foi finalizada!<BR><u>$messagem</u>",33);
            }
        }
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            list($endDate, $occurrences) = $this->calendarDetailService->setEndDateOrOccurrences($recurrenceType, $beginDate, $occurrences,
                $exceptionQuantitative, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException,
                $monthlyDay, $monthly, $orderDay, $weekDay, $yearDay);
            /** @var Entities\Project $meeting */
            $meeting = $this->projectRepository->find($id);
            $workTime = Utilities::timeToSeconds($workTime);
            /** @var Entities\Project[] $meetingPlans */
            $meetingPlans = $this->setMeetingRecurrence($name, $recurrenceType, $beginDate, $occurrences, $status, $endDate, $every, $sun, $mon, $tue, $wed,
                $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay, $weekDay, $yearDay, $workTime, $meeting, $beginTime);

            $this->entityManager->flush();
            /** @var Entities\ResourceProject $manager */
            $manager = $this->resourceProjectRepository->findOneBy(array(
                "project" => $meeting, "type" => ResourceProject::MANAGER));
            foreach ($meetingPlans as $meetingPlanning) {
                if ($manager) {
                    $this->resourceProjectService->createResourceProject($manager->getResourceC(),
                        $meetingPlanning, ResourceProject::MANAGER);
                }
                $resourceProject = $this->resourceProjectRepository->findBy(array("project" => $meeting,
                    "type" => Entities\ResourceProject::WORK));
                $resourceIds = Utilities::getArrayByArrayObjectWithGet($resourceProject,array('getResourceC','getId'));
                $this->setParticipantsMeeting($resourceIds, $meetingPlanning);
                $this->copyMeetingSchedule($meetingPlanning, $meeting);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function copyMeetingScheduleChildren(Entities\Task $task,Entities\Project $targetProject, array $createdTask,
                                                Entities\Task $newTaskParent){
        /** @var Entities\Task[] $tasks */
        $tasks = $this->taskRepository->findBy(array('parent' => $task));
        $tasks = $this->getTaskIsNotConcluded($tasks);
        foreach ($tasks as $task) {
            $scheduleResource = $this->getResourceIdByTask($task);
            $newTask = $this->taskService->createFastSchedule($task->getName(),$targetProject->getId(),$newTaskParent->getId(),
                $task->getClassification(), $task->getDescription(),$scheduleResource,$task);
            $this->copyRelationshipScheduleWithTask($task,$newTask);
            $createdTask[] = $newTask;
            $this->copyMeetingScheduleChildren($task, $targetProject,$createdTask,$newTask);
        }
        return $createdTask;
    }

    public function copyRelationshipScheduleWithTask(Entities\Task $schedule,Entities\Task $newSchedule)
    {
        /** @var Entities\TaskRelationship[] $taskRelationships */
        $taskRelationships = $this->taskRelationshipRepository->findBy(array('taskRelationship'=>$schedule,
            'type' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE));
        foreach ($taskRelationships as $taskRelationship) {
            $newTaskRelationship = $this->taskService->createTaskRelationshipMeeting($taskRelationship->getTask(),
               $newSchedule, Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE);
            $newTaskRelationship->setTaskOrder($taskRelationship->getTaskOrder());
        }
        $this->entityManager->flush();
    }

    public function scheduleReplication(Entities\Task $task){
        /** @var Entities\Task[] $taskChildren */
        $taskChildren = $task->getTaskInRelationshipInverseTypeSchedule();
        $taskChildren = $this->getTaskForProjectIsProgressOrPlanning($taskChildren);
        $labelsRelated = $this->getLabelIdByTask($task);
        foreach ($taskChildren as $taskChild) {
            $this->taskService->editSchedule($taskChild, $taskChild->getProject(), $task->getClassification(),
                $taskChild->getParent(), $task->getDescription(), $task->getName(), $task->getResourceIds(),
                $task->getTaskStatus(),$task->getCommitmentDate(), $labelsRelated);
        }
        /** @var Entities\Task[] $taskTops */
        $taskTops = $task->getTaskInRelationshipTypeSchedule();
        $taskTops = $this->getTaskForProjectIsProgressOrPlanning($taskTops);
        foreach ($taskTops as $taskTop) {
            $this->taskService->editSchedule($taskTop, $taskTop->getProject(), $task->getClassification(),
                $taskTop->getParent(), $task->getDescription(), $task->getName(), $task->getResourceIds(),
                $task->getTaskStatus(),$task->getCommitmentDate(), $labelsRelated);
            $this->scheduleReplication($taskTop);
        }
    }

    public function getLabelIdByTask ($task){
        return $this->taskService->getLabelIdByTask($task);
    }


    public function editSchedule($taskId,$projectId,$classificationId,$description,$name,$parentId,$resources = array(),
                                 $taskStatusId = null, $isSchedule = 'N', $labelsNotRelated = array(),
                                 $labelsRelated = array(),$scheduleRelationshipIds = array(), $meetingId = null,
                                 $commitmentDate = null, $priority = null)
    {
        
        $connection = $this->entityManager->getConnection();
        $reload = false;
        try {
            $connection->beginTransaction();
            /** @var Entities\Task $task */
            $task = $this->taskRepository->find($taskId);
            
            if ($isSchedule == 'Y') {
                /** @var Entities\Project $project */
                $project = $this->projectRepository->find($projectId);

                if ($classificationId) {
                    /** @var Entities\Classification $classification */
                    $classification = $this->classificationRepository->find($classificationId);
                } else {
                    $classification = null;
                }

                if ($parentId) {
                    /** @var Entities\Task $parent */
                    $parent = $this->taskRepository->find($parentId);
                } else {
                    $parent = null;
                }

                if ($taskStatusId) {
                    /** @var Entities\TaskStatus $parent */
                    $taskStatus = $this->taskStatusRepository->find($taskStatusId);
                } else {
                    $taskStatus = null;
                }

                if($priority){
                    $priority = $this->entityManager->find(Utilities::ENTITY_DIR . 'Priority', $priority);
                }
                $task = $this->taskService->editSchedule($task, $project, $classification, $parent, $description, $name,
                    $resources, $taskStatus,$commitmentDate,$labelsRelated, $priority);

                $this->scheduleReplication($task);
                $reload = $this->taskService->setInProgressMeetingWhenBelated($project);
            } else {
                $labels = array_merge($labelsRelated, $labelsNotRelated);
                $labels = array_unique($labels);
                $this->taskService->setLabels($labels, $task);
                $scheduleRelationshipTops = [];
                foreach ( $scheduleRelationshipIds as $scheduleRelationshipTaskId) {
                    /** @var Entities\Task $scheduleRelationship */
                    $scheduleRelationship = $this->taskRepository->find($scheduleRelationshipTaskId);
                    if($scheduleRelationship){
                        $scheduleRelationshipTop = $this->taskService->getScheduleTop($scheduleRelationship);
                        $scheduleRelationshipTops[] = $scheduleRelationshipTop->getId();
                    }

                }

                /** @var Entities\Task $scheduleRelationship */
                foreach ($scheduleRelationshipIds as $scheduleRelationshipTaskId) {
                    if($scheduleRelationshipTaskId) {
                        $taskRelationship = $this->getTaskRelationshipByMeeting($meetingId, $scheduleRelationshipTaskId);
                        if(!$taskRelationship){
                            $scheduleRelationship = $this->taskRepository->find($scheduleRelationshipTaskId);
                            $this->replicationSetRelationshipScheduleWithTask($task, $scheduleRelationship, $meetingId);
                            $task->setCommitmentDate($commitmentDate);
                            $this->entityManager->persist($task);
                            if ($scheduleRelationship) {
                                $reload = $this->taskService->setInProgressMeetingWhenBelated($scheduleRelationship->getProject());
                            }
                            $this->entityManager->flush();
                        }
                    }
                }
                $project = $this->projectRepository->find($meetingId);
                if($project && $project->getParent()){
                    $meetingId = $project->getParent()->getId();
                }

                $originalSchedules = $this->taskRepository->getScheduleRelationshipFromTask($taskId,$meetingId);
                foreach ($originalSchedules as $originalSchedule) {
                    /** @var Entities\Task $originalScheduleTask */
                    $originalScheduleTask = $this->taskRepository->findOneBy(array('id' =>$originalSchedule['TASK_RELATIONSHIP']));

                    if(!in_array($originalSchedule['TASK_RELATIONSHIP'], $scheduleRelationshipTops)){
                        $this->taskService->replicationRemoveRelationshipScheduleWithTask($originalScheduleTask,$task);
                        $task->setCommitmentDate($commitmentDate);
                        $this->entityManager->persist($task);
                        $this->entityManager->flush();
                    }
                }
                
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return array($task->getId(),$reload);
    }
    public function getLastOrderTaskMeeting($schedule,$type){
        $taskOrder = 1;
        /** @var Entities\TaskRelationship $taskRelationship */
        $taskRelationship = $this->taskRelationshipRepository->findOneBy(array('taskRelationship' => $schedule,
            'type' => $type),array('taskOrder'=>'DESC'));
        if($taskRelationship){
            $taskOrder = $taskRelationship->getTaskOrder();
        }
        return $taskOrder;
    }
    
    public function replicationSetRelationshipScheduleWithTask($task, $schedule = null, $meetingId = null, $taskOrder = null){
        if(!$taskOrder && $schedule){
            $taskOrder = $this->getLastOrderTaskMeeting($schedule,Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE);
        }
        $this->taskService->replicationSetRelationshipScheduleWithTask($task, $schedule, $meetingId, $taskOrder);
    }

    public function saveTaskOrder($taskId, $scheduleId, $taskOrder)
    {
        $connection = $this->entityManager->getConnection();
        try {
                $connection->beginTransaction();
            /** @var Entities\Task $schedule */
            $reload = $this->taskService->setInProgressMeetingWhenBelatedByScheduleId($scheduleId);
            $schedule = $this->taskRepository->find($scheduleId);
            $modifiedProjectDate = $schedule->getProject()->getBeginDate();
            $scheduleTop = $this->taskService->getScheduleTop($schedule);
            /** @var Entities\TaskRelationship[] $taskRelationships */
            $taskRelationships = $this->taskRelationshipRepository->findBy(array('taskRelationship' => $scheduleTop->getId(),
                'type' => Entities\TaskRelationship::SCHEDULE_TYPE));
            foreach($taskRelationships as &$taskRelationship){
                $childScheduleId = $taskRelationship->getTask()->getId();
                $projectDate = $taskRelationship->getTask()->getProject()->getBeginDate();
                if(($projectDate->format('Ymd') >= $modifiedProjectDate->format('Ymd'))) {
                    $childTaskRelationship = $this->taskRelationshipRepository->findOneBy(array('task' => $taskId, 'taskRelationship' => $childScheduleId,
                        'type' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE));
                    if($childTaskRelationship) {
                        $childTaskRelationship->setTaskOrder($taskOrder);
                        $this->entityManager->persist($childTaskRelationship);
                    }
                }
            };
            $parentTaskRelationship = $this->taskRelationshipRepository->findOneBy(array('task' => $taskId,
                'taskRelationship' => $scheduleTop->getId(),  'type' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE));
            $parentTaskRelationship->setTaskOrder($taskOrder);
            $this->entityManager->persist($parentTaskRelationship);
            $this->entityManager->flush();
            $connection->commit();
            return $reload;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveScheduleListOrder($id, $taskOrder){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $schedule = $this->taskRepository->find($id);
            $schedule->setTaskOrder($taskOrder);

            $reload = true;

            $this->entityManager->flush();
            $connection->commit();
            return $reload;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    public function setRelationshipScheduleWithTask(Entities\Task $task,Entities\Task $scheduleRelationship = null,
                                                    $meetingId = null,$taskOrder = null)
    {

       $taskRelationship = $this->getTaskRelationshipByMeeting($meetingId, $task);

        if ($scheduleRelationship) {
            if (!$taskRelationship) {
                $taskRelationship = $this->taskService->createTaskRelationshipMeeting($task, $scheduleRelationship,
                    Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE);
            }
            $taskRelationship->setTaskRelationship($scheduleRelationship);
            $this->entityManager->persist($taskRelationship);
            $this->entityManager->flush();
        }else if($taskRelationship){
            $this->entityManager->remove($taskRelationship);
            $this->entityManager->flush();
        }

    }


    /**
     * @param Entities\Task[] $tasks
     * @return mixed
     */
    protected function getPrimaryScheduleInArray( array &$tasks)
    {
        foreach ($tasks as $key => $task) {
            if (!$task->getParent()) {
                $taskPrime = $task;
                unset($tasks[$key]);
                return $taskPrime;
            }
        }
        return null;
    }

    public function copyMeetingSchedule( Entities\Project $targetProject, Entities\Project $project)
    {
        $allTaskStatusISNotConcluded = $this->taskStatusRepository->findBy(
            array('control' => self::getAllTaskStatusControlsISNotConcluded()));
        /** @var Entities\Task[] $tasks */
        $primaryScheduleTarget = $this->taskRepository->findOneBy(array("project" => $project, 'parent' => null));
        $tasks = $this->taskRepository->findBy(array(
            "project" => $project,
            'taskStatus' => $allTaskStatusISNotConcluded,
            'parent' => $primaryScheduleTarget,
        ));
        /** @var Entities\Task $primaryScheduleTarget */
        $primarySchedule = $this->taskService->copySchedule($primaryScheduleTarget,$targetProject);
        /** @var Entities\Task[] $tasks */
        foreach ($tasks as $task) {
            $newTask = $this->taskService->copySchedule($task,$targetProject,$primarySchedule);
            $this->copyRelationshipScheduleWithTask($task,$newTask);
        }
    }


    public function createMeeting($name, $status = 'A', $duration, $date, $userStructure, $projectType,
                                  $projectStatus,$description){
        $meeting = $this->projectFactory->createMeeting($name, $status, $duration, $date, $date, $userStructure, $projectType,
            $projectStatus, $description);
        $this->entityManager->persist($meeting);
        return $meeting;

    }

    public function removeMeeting($meetingId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Entities\Project $meeting */
            $meeting = $this->projectRepository->find($meetingId);
            if($meeting &&
                $meeting->getProjectType() &&
                $meeting->getProjectType()->getExternaltype() == Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING){
                $schedules = $this->taskRepository->findBy(array('project' => $meeting));
                if (count($schedules) > 1) {
                    throw new Exception(Messages::ERR_MEETING_DELETE);
                }
            }
            $this->deleteMeeting($meeting);
            /** @var Entities\Project[] $meetingPlannings */
            $meetingPlannings = $this->projectRepository->findBy(array('parent' => $meeting));
            foreach ($meetingPlannings as $meetingPlanning) {
                $this->deleteMeeting($meetingPlanning);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeSchedule($scheduleId,$removeTaskTop = true)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $reload = $this->taskService->setInProgressMeetingWhenBelatedByScheduleId($scheduleId);
            /** @var Entities\Task $task */
            $task = $this->taskRepository->find($scheduleId);
            if(!$task->isSchedule()){
                throw new \Exception(Messages::ERR_IS_NOT_SCHEDULE,3);
            }
            if($removeTaskTop){
                $taskTops = $task->getTaskInRelationshipTypeSchedule();
                if (!empty($taskTops)) {
                    $task = $taskTops[0];
                }
                /** @var Entities\Task[] $childrenTasks */
            }
            $this->deleteLogicalAllChildrenSchedule($task);
            $this->entityManager->flush();
            $connection->commit();
            return $reload;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function setMeeting($meetingProject, \DateTime $taskPeriodEnd = null,
                               \DateTime $taskPeriodStart = null, \DateTime $issuePeriodEnd = null,
                               \DateTime $issuePeriodStart = null, $allowScheduleFilter)
    {
        /** @var Entities\Meeting $meeting */
        $meeting = $this->meetingRepository->findoneBy(array('project' => $meetingProject));
        $meeting = $this->meetingFactory->saveMeeting($meeting, $meetingProject, $taskPeriodEnd, $taskPeriodStart, $issuePeriodEnd, $issuePeriodStart, $allowScheduleFilter);
        return $meeting;
    }

    public function savePrimarySchedule(Entities\Project $meetingProject)
    {
        $primarySchedule = $this->taskRepository->findOneBy(array('project' => $meetingProject, 'parent' => null));
        if (empty($primarySchedule)) {
            $primarySchedule = $this->taskService->createPrimarySchedule('Tarefa da reunião ' . $meetingProject->getName(), $meetingProject->getId(), null, null,
                'Tarefa gerado pelo PM para reunião');
        }
        $primarySchedule->setName('Tarefa da reunião ' . $meetingProject->getName());
        $this->entityManager->persist($primarySchedule);
        $this->entityManager->flush();
        return $primarySchedule;
    }

    public function setTaskLabels($taskLabelsIds, $primarySchedule)
    {
        $this->taskService->associateTaskLabelsMeeting($primarySchedule->getId(),$taskLabelsIds,Entities\TaskLabel::TYPE_TASK);
        return $primarySchedule;
    }

    public function setIssueLabels($taskLabelsIds, $primarySchedule)
    {
        $this->taskService->associateTaskLabelsMeeting($primarySchedule->getId(),$taskLabelsIds,Entities\TaskLabel::TYPE_ISSUE);
        return $primarySchedule;
    }

    public function saveMeeting($id, $name, $description, $managerId, $resourceIds, $projectTypeId, $clientIds,
                                $productIds, $projectSprintIds, $taskStatusIds, $taskLabelsIds, $taskPeriodEnd,
                                $taskPeriodStart, $issueStatusIds, $issueLabelsIds, $issuePeriodEnd, $issuePeriodStart,
                                $beginDate, $time, $projectStatusId,$duration = 3600, $allowScheduleFilter)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            /** @var Entities\ProjectType $projectType */
            list($meetingProject, $projectType) = $this->saveSetsMeeting($id, $name, $description, $projectTypeId, $beginDate, $time, $projectStatusId, $duration, $allowScheduleFilter);
            $this->setParticipantsMeeting($resourceIds, $meetingProject);
            $this->setProjectManagerRecursive($managerId, $meetingProject);
            if($projectType->getExternaltype() == Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING){
                $meeting = $this->setMeeting($meetingProject, $taskPeriodEnd, $taskPeriodStart, $issuePeriodEnd,
                    $issuePeriodStart, $allowScheduleFilter);
                $primarySchedule = $this->savePrimarySchedule($meetingProject);
                $this->entityManager->persist($meeting);
                $this->entityManager->flush();
                $this->setClientsMeeting($clientIds, $meetingProject);
                $this->setProjectSprints($projectSprintIds, $meeting);
                $this->setTaskStatus($taskStatusIds, $meeting);
                $this->setIssueStatus($issueStatusIds, $meeting);
                $this->setTaskLabels($taskLabelsIds, $primarySchedule);
                $this->setIssueLabels($issueLabelsIds, $primarySchedule);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $meetingProject;
    }


    public function setMeetingRecurrence($name, $recurrenceType, $beginDate, $occurrences, $status, $endDate, $every, $sun, $mon,
                                         $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly, $orderDay,
                                         $weekDay, $yearDay, $workTime, Entities\Project $meeting, $beginTime)
    {
        if (CalendarDetail::RECURRENCE_TYPE_DAILY == $recurrenceType) {
            $meetingPlans = $this->setRecurrenceMeetingDaily($name, $beginDate, $endDate, $every, $workTime, $meeting, $beginTime);
        } else if (CalendarDetail::RECURRENCE_TYPE_WEEKLY == $recurrenceType) {
            $meetingPlans = $this->setRecurrenceMeetingWeekly($name, $beginDate, $endDate, $every, $sun, $mon, $tue, $wed, $thu,
                $fri, $sat, $workTime, $meeting, $beginTime);
        } else if (CalendarDetail::RECURRENCE_TYPE_MONTHLY == $recurrenceType) {
            $meetingPlans = $this->setRecurrenceMeetingMonthly($name, $beginDate, $endDate, $every, $monthlyDay, $orderDay,
                $weekDay, $workTime, $meeting, $beginTime);
        } else if (CalendarDetail::RECURRENCE_TYPE_YEARLY == $recurrenceType) {
            $meetingPlans = $this->setRecurrenceMeetingYearly($name, $beginDate, $endDate, $monthly, $orderDay,
                $weekDay, $yearDay, $workTime, $meeting, $beginTime);
        } else {
            throw new \Exception('Tipo de recorrencia não encontrada');
        }

        return $meetingPlans;
    }

    public function setRecurrenceMeetingDaily($name, \DateTime $beginDate, \DateTime $endDate, $every, $workTime,
                                              Entities\Project $meeting, $beginTime)
    {
        $datesArray = $this->calendarDatesRepository->getDatesForCalendarDaily($beginDate->format(Utilities::FORMAT_DATE_DEFAULT),
            $endDate->format(Utilities::FORMAT_DATE_DEFAULT), $every, $beginDate->format('d'));
        $calendarDates = $this->createMeetingPlanning($name, $datesArray, $workTime, $meeting, $beginTime);

        return $calendarDates;
    }

    public function setRecurrenceMeetingWeekly($name, \DateTime $beginDate, \DateTime $endDate, $every, $sun, $mon,
                                               $tue, $wed, $thu, $fri, $sat, $workTime, $meeting, $beginTime)
    {
        $beginDate = $beginDate->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $endDate->format(Utilities::FORMAT_DATE_DEFAULT);
        $weekdaysString = $this->builderWeek($sun, $mon, $tue, $wed, $thu, $fri, $sat);
        $datesArray = $this->calendarDatesRepository->getDatesForCalendarWeek($beginDate, $endDate, $every,
            $weekdaysString);
        $calendarDates = $this->createMeetingPlanning($name, $datesArray, $workTime, $meeting, $beginTime);

        return $calendarDates;
    }

    public function setRecurrenceMeetingMonthly($name, \DateTime $beginDate, \DateTime $endDate, $every, $monthlyDay, $orderDay,
                                                $weekDay, $workTime, Entities\Project $meeting, $beginTime)
    {
        $month = $beginDate->format('d');
        $beginDate = $beginDate->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $endDate->format(Utilities::FORMAT_DATE_DEFAULT);
        if (empty($orderDay)) {
            $datesArray = $this->calendarDatesRepository->getDatesForMonthlyDayRecurrence($beginDate, $endDate,
                $monthlyDay, $every, $month);
        } else {
            $datesArray = $this->calendarDatesRepository->getDatesForMonthlyWeekRecurrence($beginDate, $endDate,
                $orderDay, $weekDay, $every, $month);
        }
        $calendarDates = $this->createMeetingPlanning($name, $datesArray, $workTime, $meeting, $beginTime);

        return $calendarDates;
    }

    public function setRecurrenceMeetingYearly($name, \DateTime $beginDate, \DateTime $endDate, $monthly, $orderDay,
                                               $weekDay, $yearDay, $workTime, Entities\Project $meeting, $beginTime)
    {
        $date = $beginDate->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $endDate->format(Utilities::FORMAT_DATE_DEFAULT);
        if (empty($orderDay)) {
            $datesArray = $this->calendarDatesRepository->getDatesForYearlyDayRecurrence($date, $endDate,
                $yearDay[0], $yearDay[1]);
        } else {
            $datesArray = $this->calendarDatesRepository->getDatesForYearlyWeekRecurrence($date, $endDate,
                $orderDay, $weekDay, $monthly);
        }
        $calendarDates = $this->createMeetingPlanning($name, $datesArray, $workTime, $meeting, $beginTime);

        return $calendarDates;
    }


    public function createMeetingPlanning($name, $datesArray, $dailyHours, Entities\Project $meeting, $beginTime)
    {
        $meetingPlans = array();
        $parent = $meeting;
        $status = $meeting->getStatus();
        $group = $meeting->getStructure();
        $dateExceptionType = RestrictionFactory::SHOULD_START;
        $projectType = Utilities::getProjectTypeMeetingOccurrences();
        if(!$projectType){
            throw new \Exception(Messages::ERR_NOT_TYPE_PROJECT_MEETING_OCCURRENCES ,2);
        }
        $projectStatus = Utilities::getProjectStatusPlanning();

        foreach ($datesArray as $dateArray) {
            $date = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_BY_BD, $dateArray['DATA']);
            $date = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_BY_BD,
                $date->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE_AMERICAN) . $beginTime . ':00');

            $meetingPlanning = $this->projectFactory->createMeetingOccurrences($name, $parent, $status, $dailyHours, $date, $date, $group,
                $dateExceptionType, $date, $projectType, $projectStatus);
            $this->entityManager->persist($meetingPlanning);

            array_push($meetingPlans, $meetingPlanning);
        }
        return $meetingPlans;
    }


    public static function builderWeek($sun, $mon, $tue, $wed, $thu, $fri, $sat)
    {
        $weeks = array(null, $sun, $mon, $tue, $wed, $thu, $fri, $sat);
        $weekNumbers = array();
        foreach ($weeks as $numberWeek => $week) {
            if ($week == CalendarDetail::WEEK_TRUE) {
                $weekNumbers[] = $numberWeek;
            }
        }
        return implode($weekNumbers, ',');
    }

    public function findSprints($projectIds = array(0))
    {
        $projectsAll = $this->projectRepository->findSprints($this->environment->getCurrentOrganizationId(), $projectIds,
            $this->environment->getCurrentUserId());
        $projectsAllIDs = array();
        foreach ($projectsAll as $project) {
            array_push($projectsAllIDs, $project['ID']);
        }
        foreach ($projectsAll as &$project) {
            if (!in_array($project['PARENT'], $projectsAllIDs)) {
                $project['PARENT'] = null;
            }
        }
        return $projectsAll;
    }
    public function getMeetingByProject(Entities\Project $project){

        /** @var Entities\Meeting $meeting */
        $meeting =  $this->meetingRepository->findOneBy(array('project'=>$project));
        if(!$meeting){
            $meeting = $this->getMeetingByProject($project->getParent());
        }
        return $meeting;
    }

    public function getProjectIdsTask (array $meetingProjects)
    {
        $projectIdTasks = array();
        if (!empty($meetingProjects)) {
            foreach ($meetingProjects as $key => $meetingProject) {
                $project = $meetingProject->getProject();
                $projectIdTasks[] = $project->getId();
            }
        }
        return $projectIdTasks;
    }

    public function getProjectIdsIssue(array $meetingProjects)
    {
        $projectIdIssues = array();
        if (!empty($meetingProjects)) {
            foreach ($meetingProjects as $key => $meetingProject) {
                $project = $meetingProject->getProject();
                $projectIdIssues[] = $project->getId();
            }
        }
        return $projectIdIssues;
    }

    /**
     * @param $projectId
     * @return array
     */
    public function getFollowerInMeeting($projectId)
    {
        $resourceProjects = $this->resourceProjectRepository->getFollowerInMeeting($projectId);
        return $resourceProjects;
    }


    public function getNextTaskOrderByTypeAndSchedule($scheduleId,$type)
    {
        $isIssue = $type == 'I' ? Entities\Task::IS_ISSUE : Entities\Task::NOT_IS_ISSUE;
        /** @var Entities\TaskRelationship[] $taskRelationships */
        $taskRelationships = $this->taskRelationshipRepository->findBy(array('taskRelationship' => $scheduleId,
            'type' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE));
        $maxTaskOrder = 0;
        foreach ($taskRelationships as $taskRelationship) {
            if ($taskRelationship->getTask()->getIsIssue() == $isIssue &&
                $taskRelationship->getTaskOrder() > $maxTaskOrder) {
                $maxTaskOrder = $taskRelationship->getTaskOrder();
            }
        }
        return [$maxTaskOrder + 1];
    }

    public function findSchedule($taskId,$projectId)
    {
        /** @var Entities\Project $project */
        $project = $this->projectRepository->find($projectId);
        $schedule = $this->getPrimaryScheduleForMeeting($project);
        $tasks = array();
        if($schedule){
            $tasks =  $this->taskRepository->findScheduleForm($taskId,$projectId);
            foreach ($tasks as &$task) {
                $task['LABELS'] = Utilities::setSelectMultiInQuery($task['LABELS']);
                list($task['LABELS_RELATED_USER'], $task['LABELS_NOT_RELATED_USER']) =
                    $this->divideLabelsByTask($task['LABELS'],$schedule,$task['TYPE_LABEL']);
                $task['MEETING_ID'] = $schedule->getProject()->getId();
                if($task['IS_SCHEDULE'] == 'Y'){
                    $task['LABELS_RELATED_USER'] = $task['LABELS_NOT_RELATED_USER'];
                    $task['LABELS_NOT_RELATED_USER'] = [];
                }
            }

        }
        return $tasks;

    }
    public function findScheduleByMeeting($projectId)
    {
        /** @var Entities\Project $project */
        $project = $this->projectRepository->find($projectId);
        $tasks = [];
        if($project){
            $tasks = $this->taskRepository->findScheduleByMeeting($project->getId());
        }

        return $tasks;

    }

    public function divideLabelsByTask($labelsIds,  Entities\Task $schedule,$typeLabel = null,$related = array(), $notRelated = array())
    {
        foreach ($labelsIds as $labelId) {
            /** @var Entities\TaskLabel $labelEntity */
            $taskLabel = $this->taskLabelRepository->findOneBy(array('label' => $labelId, 'task' => $schedule, 'type'=> $typeLabel));
            if ($taskLabel) {
                $related[] = $labelId;
            } else {
                $notRelated[] = $labelId;
            }

        }
        return array($related, $notRelated);
    }

    public function findAttachmentMeeting($projectId)
    {
        return $this->attachmentRepository->getDataAttachmentMeeting($projectId);
    }

    public function findTaskListSchedule($scheduleId, $filter = array())
    {
        $conditions = Utilities::createQueryFiltered($filter);
        return $this->taskRepository->findTaskListSchedule($scheduleId, $conditions);
    }
    
    public function findIssueListSchedule($scheduleId, $filter = array(), $page, $pageSize)
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditions = Utilities::createQueryFiltered($filter);
        return $this->taskRepository->findIssueListSchedule($scheduleId, $conditions, $limit, $offset);
    }
    
    public function findIssueListScheduleFromProject($projectIds, $scheduleId, $filter = array(), $page, $pageSize,$meetingProjectId = 0)
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditions = Utilities::createQueryFiltered($filter);
        /** @var Entities\Project $project */
        $meetingProject = $this->projectRepository->findOneBy(array('id' => $meetingProjectId));
        if($meetingProject && $meetingProject->getParent()){
            $meetingProjectId = $meetingProject->getParent()->getId();
        }
        return $this->taskRepository->findIssueListScheduleFromProject($projectIds,$scheduleId, $conditions, $limit, $offset,$meetingProjectId);
    }

    public function saveIssueListScheduleFromProject($scheduleId, $listIssueToScheduleIds, $labels = null)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $schedule = $this->taskRepository->find($scheduleId);
            foreach ($listIssueToScheduleIds as $listIssueToScheduleId) {
                $issue = $this->taskRepository->find($listIssueToScheduleId);
                $this->taskService->replicationSetRelationshipScheduleWithTask($issue, $schedule, null, '1');

                if($labels) {
                    $this->taskService->relateLabelToIssue($issue, $labels);
                };
            }
            $connection->commit();
            return 0;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findProductMeeting($projectId, $name)
    {
        /** @var Entities\Project $project */
        $project = $this->projectRepository->find($projectId);
        if (!empty($project)){
            if($project->getParent()){
                $project = $project->getParent();
            }
            $meeting =  $this->meetingRepository->findOneBy(array('project'=>$project));
            /** @var Entities\MeetingProject[] $meetingProjects */
            $meetingProjects = $this->meetingProjectRepository->findBy(array('meeting'=>$meeting));
            $projectIds = array();
            if(!empty($meetingProjects)) {
                foreach ($meetingProjects as $key => $meetingProject) {
                    $projectIds[]['ID'] = $meetingProject->getProject()->getId();
                }
            }
            $taskAll = $this->projectRepository->findProductMeeting($projectIds, $name);
            $taskAll = Utilities::fixParentsForTreeGrid($taskAll);
        } else {
            $taskAll = array();
        }
        
        return $taskAll;
    }

    public function findProjectMeeting($projectId, $conditions = array())
    {
        /** @var Entities\Project $project */
        $project = $this->projectRepository->find($projectId);
        if (!empty($project)){
            if($project->getParent()){
                $project = $project->getParent();
            }
            $meeting =  $this->meetingRepository->findOneBy(array('project'=>$project));
            /** @var Entities\MeetingProject[] $meetingProjects */
            $meetingProjects = $this->meetingProjectRepository->findBy(array('meeting' => $meeting));
            $projectIds = Utilities::getArrayByArrayObjectWithGet($meetingProjects,array('getProject','getId'));
            $userId = $this->environment->getCurrentUserId();

            $conditionsString = Utilities::createQueryFiltered($conditions);

            $taskAll = $this->projectRepository->findProjectMeeting($projectIds,$userId,$conditionsString);
            $taskAll = Utilities::fixParentsForTreeGrid($taskAll);
        } else {
            $taskAll = array();
        }
        return $taskAll;
    }

    public function findAllMeeting($conditions)
    {

        $organizationId = $this->environment->getCurrentOrganizationId();
        $projectsAll = $this->projectRepository->findAllMeeting($organizationId, $conditions);
        $projectsAll = Utilities::fixParentsForTreeGrid($projectsAll);

        return $projectsAll;
    }

    public function findLabelsByMeeting($meetingId,$typeTaskLabel)
    {
        $scheduleId = 0;

        if ($meetingId) {
            /** @var Entities\Project $project */
            $project = $this->projectRepository->find($meetingId);
            if ($typeTaskLabel == Entities\Task::MEETING_SCHEDULE_TYPE) {
                if ($project->getParent()) {
                    $project = $project->getParent();
                }
                $labels = $this->labelRepository->findMeetingLabelsForSchedule($project->getId());
            } else {
                $scheduleId = $this->getPrimaryScheduleForMeeting($project)->getId();
                $labels = $this->labelRepository->findLabelsByTask($scheduleId, $typeTaskLabel);
            }
        }else{
            $labels = $this->labelRepository->findLabelsByTask($scheduleId, $typeTaskLabel);
        }

        return $labels;
    }

    public function findInfoMeeting($projectId)
    {
        $projectArray = $this->projectRepository
            ->getDataSetForEditInfoMeeting($projectId, $this->environment->getCurrentOrganizationId(),$this->environment->getCurrentUserId());

        foreach ($projectArray as &$project) {
            $project['RESOURCE']        = EntitiesUtil::setSelectMultiInQuery($project['RESOURCE'], true);
            $project['CLIENT']          = EntitiesUtil::setSelectMultiInQuery($project['CLIENT'], true);
            $project['PROJECT_SPRINT']  = EntitiesUtil::setSelectMultiInQuery($project['PROJECT_SPRINT']);
            $project['TASK_STATUS']     = EntitiesUtil::setSelectMultiInQuery($project['TASK_STATUS']);
            $project['ISSUE_STATUS']    = EntitiesUtil::setSelectMultiInQuery($project['ISSUE_STATUS']);
            $project['TASK_LABELS']     = EntitiesUtil::setSelectMultiInQuery($project['TASK_LABELS']);
            $project['ISSUE_LABELS']    = EntitiesUtil::setSelectMultiInQuery($project['ISSUE_LABELS']);
            $project['ISSUE_PERIOD']    = EntitiesUtil::setPeriodInQuery($project['ISSUE_PERIOD_start'],$project['ISSUE_PERIOD_end']);
            $project['TASK_PERIOD']     = EntitiesUtil::setPeriodInQuery($project['TASK_PERIOD_start'],$project['TASK_PERIOD_end']);
            $project['TOTAL_TIME']      = Utilities::secondsToTime($project['TOTAL_TIME']);
        }

        return $projectArray;
    }

    /**
     * @param $resourceIds
     * @param $meetingProject
     * @throws \Exception
     */
    protected function setParticipantsMeeting($resourceIds, $meetingProject)
    {
        foreach ($resourceIds as $eachResource) {
            if (!empty($eachResource)) {
                $resource = $this->resourceRepository->find($eachResource);
                /** @var Entities\ResourceProject $resourceProject */
                $resourceProject = $this->resourceProjectRepository->findOneBy(array("project" => $meetingProject,
                    "resourceC" => $resource, "type" => Entities\ResourceProject::WORK));
                if (empty($resourceProject)) {
                    $this->createResourceProject($resource, $meetingProject, Entities\ResourceProject::WORK, true);
                }
            }
        }
        /** @var Entities\ResourceProject[] $resourceProjects */
        $resourceProjects = $this->resourceProjectRepository->findBy(array("project" => $meetingProject));
        foreach ($resourceProjects as $resourceProject) {
            if ($resourceProject->getType() == ResourceProject::WORK) {
                if (!in_array($resourceProject->getResourceC()->getId(), $resourceIds)
                ) {
                    $this->entityManager->remove($resourceProject);
                }
            }
        }
        $this->entityManager->flush();
    }

    /**
     * @param $clientIds
     * @param $meetingProject
     * @throws \Exception
     */
    protected function setClientsMeeting($clientIds, $meetingProject)
    {
        foreach ($clientIds as $eachResource) {
            if (!empty($eachResource)) {
                $resource = $this->resourceRepository->find($eachResource);
                /** @var Entities\ResourceProject $resourceProject */
                $resourceProject = $this->resourceProjectRepository->findOneBy(array("project" => $meetingProject,
                    "resourceC" => $resource, "type" => ResourceProject::SPONSOR));
                if (empty($resourceProject)) {
                    $this->createResourceProject($resource, $meetingProject, ResourceProject::SPONSOR, true);
                }
            }
        }
        /** @var Entities\ResourceProject[] $resourceProjects */
        $resourceProjects = $this->resourceProjectRepository->findBy(array("project" => $meetingProject,
            'type' => ResourceProject::SPONSOR));
        foreach ($resourceProjects as $resourceProject) {
            if (!in_array($resourceProject->getResourceC()->getId(), $clientIds)
            ) {
                $this->entityManager->remove($resourceProject);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param $productIds
     * @param $meeting
     * @throws \Exception
     */
    protected function setProductsMeeting($productIds, $meeting)
    {
        foreach ($productIds as $productId) {
            if (!empty($productId)) {
                /** @var Entities\Project $project */
                $project = $this->projectRepository->find($productId);
                /** @var Entities\MeetingProject $meetingProject */
                $meetingProject = $this->meetingProjectRepository->findOneBy(array("meeting" => $meeting,
                    "project" => $project));
                if (empty($meetingProject) && $project->isProduct()) {
                    $this->createMeetingProject($project, $meeting);
                }
            }
        }
        $this->entityManager->flush();

        /** @var Entities\MeetingProject[] $meetingProjects */
        $meetingProjects = $this->meetingProjectRepository->findBy(array("meeting" => $meeting));
        foreach ($meetingProjects as $meetingProject) {
            if ($meetingProject->getProject()->isProduct() &&
                !in_array($meetingProject->getProject()->getId(), $productIds)
            ) {
                $this->entityManager->remove($meetingProject);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param $projectIds
     * @param $meeting
     * @throws \Exception
     */
    protected function setProjectSprints($projectIds, $meeting)
    {
        foreach ($projectIds as $productId) {
            if (!empty($productId)) {
                /** @var Entities\Project $project */
                $project = $this->projectRepository->find($productId);
                /** @var Entities\MeetingProject $meetingProject */
                $meetingProject = $this->meetingProjectRepository->findOneBy(array("meeting" => $meeting,
                    "project" => $project));
                if (empty($meetingProject)) {
                    $this->createMeetingProject($project, $meeting);
                }
            }
        }
        $this->entityManager->flush();

        /** @var Entities\MeetingProject[] $meetingProjects */
        $meetingProjects = $this->meetingProjectRepository->findBy(array("meeting" => $meeting));
        foreach ($meetingProjects as $meetingProject) {
            if ( !in_array($meetingProject->getProject()->getId(), $projectIds)) {
                $this->entityManager->remove($meetingProject);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param $taskStatusIds
     * @param $meeting
     * @throws \Exception
     */
    protected function setTaskStatus($taskStatusIds, $meeting)
    {
        foreach ($taskStatusIds as $productId) {
            if (!empty($productId)) {
                /** @var Entities\TaskStatus $taskStatus */
                $taskStatus = $this->taskStatusRepository->find($productId);
                /** @var Entities\MeetingTaskStatus $meetingTaskStatus */
                $meetingTaskStatus = $this->meetingTaskStatusRepository->findOneBy(array("meeting" => $meeting,
                    "taskStatus" => $taskStatus, 'type' => Entities\MeetingTaskStatus::TYPE_TASK));
                if (empty($meetingTaskStatus)) {
                    $this->createMeetingTaskStatus($taskStatus, $meeting, Entities\MeetingTaskStatus::TYPE_TASK );
                }
            }
        }
        $this->entityManager->flush();

        /** @var Entities\MeetingTaskStatus[] $meetingProjects */
        $meetingProjects = $this->meetingTaskStatusRepository->findBy(array("meeting" => $meeting,
            'type' => Entities\MeetingTaskStatus::TYPE_TASK));
        foreach ($meetingProjects as $meetingTaskStatus) {
            if (!in_array($meetingTaskStatus->getTaskStatus()->getId(), $taskStatusIds)) {
                $this->entityManager->remove($meetingTaskStatus);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param $taskStatusIds
     * @param $meeting
     * @throws \Exception
     */
    protected function setIssueStatus($taskStatusIds, $meeting)
    {
        foreach ($taskStatusIds as $productId) {
            if (!empty($productId)) {
                /** @var Entities\TaskStatus $taskStatus */
                $taskStatus = $this->taskStatusRepository->find($productId);
                /** @var Entities\MeetingTaskStatus $meetingTaskStatus */
                $meetingTaskStatus = $this->meetingTaskStatusRepository->findOneBy(array("meeting" => $meeting,
                    "taskStatus" => $taskStatus, 'type' => Entities\MeetingTaskStatus::TYPE_ISSUE ));
                if (empty($meetingTaskStatus)) {
                    $this->createMeetingTaskStatus($taskStatus, $meeting,Entities\MeetingTaskStatus::TYPE_ISSUE);
                }
            }
        }
        $this->entityManager->flush();

        /** @var Entities\MeetingTaskStatus[] $meetingProjects */
        $meetingProjects = $this->meetingTaskStatusRepository->findBy(array("meeting" => $meeting,
            'type' => Entities\MeetingTaskStatus::TYPE_ISSUE));
        foreach ($meetingProjects as $meetingTaskStatus) {
            if (!in_array($meetingTaskStatus->getTaskStatus()->getId(), $taskStatusIds)) {
                $this->entityManager->remove($meetingTaskStatus);
                $this->entityManager->flush();
            }
        }
    }

    protected function createMeetingProject($project, $meeting)
    {
        $meetingProject = $this->meetingProjectFactory->saveMeetingProject(null, $project, $meeting);
        $this->entityManager->persist($meetingProject);
    }

    protected function createMeetingTaskStatus($taskStatus, $meeting, $type)
    {
        $meetingProject = $this->meetingTaskStatusFactory->saveMeetingTaskStatus(null, $taskStatus, $meeting, $type);
        $this->entityManager->persist($meetingProject);
    }

    protected function setProjectManagerRecursive($managerId, Entities\Project $meeting)
    {
        /** @var Entities\Resource $managerResource */
        $managerResource = $this->resourceRepository->find($managerId);
        $this->resourceProjectService->updateProjectPartnerSponsorOrManager($managerResource, $meeting,
                                                                    ResourceProject::MANAGER, true, true);
    }

    /**
     * @param Entities\MeetingProject[] $meetingProjects
     * @param Entities\Meeting $meeting
     * @param $projectId
     * @return array
     */
    protected function getAllFilters(array $meetingProjects,Entities\Meeting $meeting, $projectId)
    {
        $projectIdTasks = array();
        $projectIdIssues = array();
        list($projectIdTasks, $projectIdIssues) = $this->getProjectIdsFilter($meetingProjects, $projectIdTasks, $projectIdIssues);

        $betweenDateTask = $this->getBetweenDateTaskFilter($meeting);

        $betweenDateIssue = $this->getBetweenDateIssueFilter($meeting);

        $meetingTaskStatusIssue = $this->getMeetingTaskStatusIssueFilter($meeting);

        $meetingTaskStatusTask = $this->getMeetingTaskStatusTaskFilter($meeting);

        $resourceProjectSponsors = $this->getResourceProjectSponsorsFilter($projectId);

        $taskLabels = $this->getTaskLabelsFilter($meeting);

        $issueLabels = $this->getIssueLabelsFilter($meeting);

        return array($projectIdTasks, $projectIdIssues, $betweenDateTask, $betweenDateIssue, $meetingTaskStatusIssue,
            $meetingTaskStatusTask, $resourceProjectSponsors, $taskLabels, $issueLabels);
    }

    public function getPrimarySchedule($scheduleProject){
        return $this->taskService->getPrimarySchedule($scheduleProject);
    }
    /**
     * @param Entities\Task[] $tasks
     * @return array
     */
    protected function getTaskIsNotConcluded(array $tasks)
    {
        $taskIds = array();
        foreach ($tasks as $task) {
            if ($task && $task->getTaskStatus() && $task->getTaskStatus()->getControl() != Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED) {
                $taskIds[] = $task;
            }else if(!$task->getTaskStatus()){
                $taskIds[] = $task;
            }
        }
        return $taskIds;
    }
    /**
     * @param Entities\Task[] $tasks
     * @return Entities\Task[]
     */
    protected function getTaskForProjectIsProgressOrPlanning(array $tasks)
    {
        return $this->taskService->getTaskForProjectIsProgressOrPlanning($tasks);
    }

    public function createResourceProject($resource, $project, $type = ResourceProject::WORK, $recursive = false) {
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            if(empty($type)){
                $type = ResourceProject::WORK;
            }

            $project = $this->projectRepository->find($project);
            $resource = $this->resourceRepository->find($resource);
            if($recursive){

                /** @var Entities\Project[] $children */
                $children = $this->projectRepository->getProjectChildrenNotConcludedOrCanceled($project->getId());

                foreach ($children as $child) {
                    $this->createResourceProject($resource->getId(), $child, ResourceProject::WORK, false,true);
                }
            }
            $entity = $this->resourceProjectRepository->findOneBy(array('resourceC' => $resource,'project' => $project,
            'type' => $type));
            if(empty($entity)){
                $entity = $this->resourceProjectFactory->createResourceProject($resource, $project, $type);
            }
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $task
     * @return array
     */
    protected function getResourceIdByTask($task)
    {
        $scheduleResource = $this->allocationRepository->findBy(array('task' => $task));
        $scheduleResource = Utilities::getArrayByArrayObjectWithGet($scheduleResource, array('getResource','getId'));
        return $scheduleResource;
    }

    /**
     * @param $projectId
     * @return array
     */
    protected function getResourceProjectSponsorsFilter($projectId)
    {
        $resourceProjectSponsors = $this->resourceProjectRepository->findBy(array("project" => $projectId,
            'type' => ResourceProject::SPONSOR));
        $resourceProjectSponsors = Utilities::getArrayByArrayObjectWithGet($resourceProjectSponsors,
            array('getResourceC', 'getId'));
        return $resourceProjectSponsors;
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array
     */
    protected function getMeetingTaskStatusTaskFilter(Entities\Meeting $meeting)
    {
        /** @var Entities\MeetingTaskStatus[] */
        $meetingTaskStatusTask = $this->meetingTaskStatusRepository->findBy(array("meeting" => $meeting,
            'type' => Entities\MeetingTaskStatus::TYPE_TASK));
        $meetingTaskStatusTask = Utilities::getArrayByArrayObjectWithGet($meetingTaskStatusTask,
            array('getTaskStatus', 'getId'));
        return $meetingTaskStatusTask;
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array
     */
    protected function getMeetingTaskStatusIssueFilter(Entities\Meeting $meeting)
    {
        /** @var Entities\MeetingTaskStatus[] */
        $meetingTaskStatusIssue = $this->meetingTaskStatusRepository->findBy(array("meeting" => $meeting,
            'type' => Entities\MeetingTaskStatus::TYPE_ISSUE));
        $meetingTaskStatusIssue = Utilities::getArrayByArrayObjectWithGet($meetingTaskStatusIssue,
            array('getTaskStatus', 'getId'));
        return $meetingTaskStatusIssue;
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array|string
     */
    protected function getBetweenDateIssueFilter(Entities\Meeting $meeting)
    {
        $betweenDateIssue = '';
        if ($meeting->getIssueDateStart()) {
            $betweenDateIssue = array($meeting->getIssueDateStart()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                $meeting->getIssueDateEnd()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE));
            return $betweenDateIssue;
        }
        return $betweenDateIssue;
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array|string
     */
    protected function getBetweenDateTaskFilter(Entities\Meeting $meeting)
    {
        $betweenDateTask = '';
        if ($meeting->getTaskDateStart()) {
            $betweenDateTask = array($meeting->getTaskDateStart()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                $meeting->getTaskDateEnd()->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE));
            return $betweenDateTask;
        }
        return $betweenDateTask;
    }

    /**
     * @param Entities\Project[] $meetingProjects
     * @param $projectIdTasks
     * @param $projectIdIssues
     * @return array
     */
    protected function getProjectIdsFilter(array $meetingProjects, $projectIdTasks, $projectIdIssues)
    {
        if (!empty($meetingProjects)) {
            foreach ($meetingProjects as $key => $meetingProject) {
                $project = $meetingProject->getProject();
                $projectIdTasks[] = $project->getId();
                $projectIdIssues[] = $project->getId();
            }
        }
        return array($projectIdTasks, $projectIdIssues);
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array
     */
    protected function getTaskLabelsFilter(Entities\Meeting $meeting)
    {
        $taskLabels = $this->taskLabelRepository->findBy(array('task' => $this->getPrimarySchedule($meeting->getProject()->getId()), 'type' => Entities\TaskLabel::TYPE_TASK));
        $taskLabels = Utilities::getArrayByArrayObjectWithGet($taskLabels, array('getLabel', 'getId'));
        return $taskLabels;
    }

    /**
     * @param Entities\Meeting $meeting
     * @return array
     */
    protected function getIssueLabelsFilter(Entities\Meeting $meeting)
    {
        $issueLabels = $this->taskLabelRepository->findBy(array('task' => $this->getPrimarySchedule($meeting->getProject()->getId()), 'type' => Entities\TaskLabel::TYPE_ISSUE));
        $issueLabels = Utilities::getArrayByArrayObjectWithGet($issueLabels, array('getLabel', 'getId'));
        return $issueLabels;
    }

    /**
     * @param Entities\Task $task
     */
    protected function deleteLogicalAllScheduleRelationship(Entities\Task $task)
    {

        /** @var Entities\Task[] $taskChildren */
        $taskChildren = $task->getTaskInRelationshipInverseTypeSchedule();
        $taskChildren = $this->getTaskForProjectIsProgressOrPlanning($taskChildren);
        foreach ($taskChildren as $taskChild) {
            $this->taskService->deleteTask($taskChild);
            $this->entityManager->persist($taskChild);
        }
        $this->taskService->deleteTask($task);
        $this->entityManager->persist($task);
    }

    /**
     * @param $task
     */
    protected function deleteLogicalAllChildrenSchedule($task)
    {
        $childrenTasks = $this->taskRepository->findBy(array('parent' => $task->getId()));
        if (!empty($childrenTasks)) {
            foreach ($childrenTasks as $childTask) {
                $this->deleteLogicalAllChildrenSchedule($childTask);
            }
        }
        $this->deleteLogicalAllScheduleRelationship($task);
    }

    /**
     * @param $meeting
     * @return array
     * @throws \Exception
     */
    protected function deleteMeeting(Entities\Project $meeting)
    {
        $labels = $this->labelRepository->findBy(array('project' => $meeting));
        $taskLabels = $this->taskLabelRepository->findBy(array('label' => $labels));

        Utilities::removeArrayEntities($this->entityManager,$taskLabels);
        Utilities::removeArrayEntities($this->entityManager,$labels);

        $comments = $this->commentsRepository->findBy(array('project' => $meeting));

        Utilities::removeArrayEntities($this->entityManager,$comments);

        $this->projectService->deleteAllAttachmentsByProject($meeting);

        $scheduleIds = $this->taskRepository->findBy(array('project' => $meeting));
        foreach ($scheduleIds as $scheduleId) {
            $this->removeSchedule($scheduleId,false);
        }
        $meeting->setStatus(LogicalDeleted::STATUS_LOGICAL_DELETED);
        $this->entityManager->persist($meeting);
    }

    /**
     * @param $id
     * @param $name
     * @param $description
     * @param $projectTypeId
     * @param $beginDate
     * @param $time
     * @param $projectStatusId
     * @param $duration
     * @return array
     * @throws \Exception
     */
    protected function saveSetsMeeting($id, $name, $description, $projectTypeId, $beginDate, $time, $projectStatusId, $duration)
    {
        if ($id) {
            /** @var Entities\Project $meetingProject */
            $meetingProject = $this->projectRepository->find($id);
            $meetingProject->setDescription($description);
            $meetingProject->setName($name);
            $meetingProject->setDuration($duration);
            $projectType = $this->projectTypeRepository->find($projectTypeId);
            if ($beginDate && $time) {
                $date = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $beginDate . $time . ':00');
                $meetingProject->setBeginDate($date);
                $meetingProject->setEndDate($date);
            }
            if ($projectStatusId) {
                $projectStatus = $this->taskStatusRepository->find($projectStatusId);
            } else {
                $projectStatus = Utilities::getProjectStatusPlanning();
            }

            $this->setTaskStatusInMeeting($projectStatus, $meetingProject);

            $this->entityManager->persist($meetingProject);
            $this->entityManager->flush();
        } else {
            $date = new \DateTime();
            $user = $this->environment->getCurrentUser();
            $userStructure = $this->resourceService->getStructureForResource($user);
            $projectType = Utilities::getProjectTypeMeeting();
            $projectStatus = Utilities::getProjectStatusPlanning();
            if (!$projectType) {
                throw new \Exception(Messages::ERR_NOT_TYPE_PROJECT_MEETING, 1);
            }
            $meetingProject = $this->createMeeting($name, 'A', $duration, $date, $userStructure, $projectType,
                $projectStatus, $description);
            $this->entityManager->flush();

        }
        return array($meetingProject, $projectType);
    }


    /**
     * @param Entities\Project $project
     * @return Entities\Task
     */
    protected function getPrimaryScheduleForMeeting(Entities\Project $project)
    {
        if ($project->getParent()) {
            $project = $project->getParent();
        }
        $schedule = $this->getPrimarySchedule($project);
        return $schedule;
    }

    /**
     * @param $meetingId
     * @param Entities\Task $task
     * @return Entities\TaskRelationship
     */
    protected function getTaskRelationshipByMeeting($meetingId, $task)
    {
        $this->taskService->getTaskRelationshipByMeeting($meetingId, $task);
    }

    public function findRecordsOfMeeting($scheduleId,$projectId,$filters)
    {

        $scheduleIds = array();
        $projectIds = array();
        if($scheduleId){
            /** @var Entities\Task $schedule */
            $schedule = $this->taskRepository->find($scheduleId);
            if($schedule) {
                $scheduleTop = $this->taskService->getScheduleTop($schedule);
                if ($scheduleTop->getId() == $schedule->getId()) {
                    $schedules = $scheduleTop->getTaskInRelationshipInverseTypeSchedule();
                    $scheduleIds = Utilities::getArrayByArrayObjectWithGet($schedules);
                    $scheduleIds[] = $scheduleTop->getId();
                } else {
                    $scheduleIds[] = $scheduleId;
                }
            }else{
                $scheduleIds[] = 0;
            }
        }
        if($projectId){
            $project = $this->projectRepository->find($projectId);
            $childrenProjects = $this->projectRepository->findBy(array('parent' => $project));
            $projectIds = Utilities::getArrayByArrayObjectWithGet($childrenProjects);
            $projectIds[] = $project->getId();
        }
        return $this->commentsRepository->findRecordsOfMeeting($scheduleIds,$projectIds,$filters);
    }

    /**
     * @param Entities\TaskStatus $projectStatus
     * @param  Entities\Project $meetingProject
     */
    protected function setTaskStatusInMeeting($projectStatus, $meetingProject)
    {

        $meeting = $meetingProject->getParent();
        $statusImmutable = array(
            Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
        );
        if ($meeting) {
            $meetingProjectStatus = $meeting->getProjectStatus();
            if (
                in_array($meetingProjectStatus->getControl(), $statusImmutable) &&
                !in_array($projectStatus->getControl(), $statusImmutable)
            ) {
                throw new Exception('TaskStatus não pode ser mudado', 4);
            }
        } else if (in_array($projectStatus->getControl(), $statusImmutable)) {
            /** @var Entities\Project[] $meetingOccurrences */
            $meetingOccurrences = $this->projectRepository->findBy(array('parent' => $meetingProject));
            foreach ($meetingOccurrences as $meetingOccurrence) {
                if ($meetingOccurrence->getProjectStatus() &&
                    !in_array($meetingOccurrence->getProjectStatus()->getControl(), $statusImmutable)
                ) {
                    $meetingOccurrence->setProjectStatus($projectStatus);
                    $this->entityManager->persist($meetingOccurrence);
                }
            }

        }
        $meetingProject->setProjectStatus($projectStatus);
    }

    public function findScheduleMeetingPlanning($projectId, $param, $page, $pageSize)
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filter = EntitiesUtil::createQueryFiltered($param);
        return $this->taskRepository->findScheduleMeetingPlanning($projectId, $filter, $limit, $offset);
    }

    public function findTaskMeetingPlannig($projectId, $param, $page, $pageSize)
    {
        try {
            $limit = $page * $pageSize;
            $offset = ($page - 1) * $pageSize;

            $project = $this->projectRepository->find($projectId);
            $isOccurrences = !!$project->getParent();
            $meeting = $this->getMeetingByProject($project);
            $meetingProjects = $this->meetingProjectRepository->findBy(array('meeting' => $meeting));
            $isAllowScheduleFilter = $meeting->getAllowScheduleFilter();
            list($projectIdFilter, $meetingTaskStatusFilter, $betweenDateFilter, $taskLabelsFilter, $taskScheduleFilter) = $this->createFiltersFindTaskMeetingPlannig($meetingProjects, $meeting, $projectId, $isAllowScheduleFilter);
            $scheduleFilter = $projectIdFilter . $meetingTaskStatusFilter . $betweenDateFilter . $taskLabelsFilter;

            if(empty($betweenDateFilter) && empty($meetingTaskStatusFilter) && $isAllowScheduleFilter == 'Y'){
                $scheduleFilter = '';
                $taskScheduleFilter = str_replace("OR", "AND" , $taskScheduleFilter);
            }else if(empty($betweenDateFilter) && empty($meetingTaskStatusFilter) && $isAllowScheduleFilter == 'N'){
                return [];
            } else if ((!empty($betweenDateFilter) || !empty($meetingTaskStatusFilter)) && $isAllowScheduleFilter == 'N'){
                $taskScheduleFilter = '';
            }
            $filter = EntitiesUtil::createQueryFiltered($param);
            return $this->taskRepository->findTaskMeetingPlannig($scheduleFilter, $taskScheduleFilter, $filter, $limit, $offset);
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    public function createFiltersFindTaskMeetingPlannig(array $meetingProjects,Entities\Meeting $meeting, $projectId, $isAllowScheduleFilter)
    {
        $projectIds = $this->getProjectIdsTask($meetingProjects);
        $betweenDateTask = $this->getBetweenDateTaskFilter($meeting);
        $meetingTaskStatus = $this->getMeetingTaskStatusTaskFilter($meeting);
        $resourceProjectSponsors = $this->getResourceProjectSponsorsFilter($projectId);
        $taskLabels = $this->getTaskLabelsFilter($meeting);

        $taskScheduleFilter = '';
        if($isAllowScheduleFilter = 'Y'){
            $schedulesIds = $this->taskRepository->findSchedulesIdsByMeeting($projectId);
            if(!empty($schedulesIds)){
                $schedulesIds = EntitiesUtil::getArrayFromArrayAssociative($schedulesIds);
                $schedulesIdsString = implode(',', $schedulesIds);
                $taskScheduleFilter = "OR TAS.ID IN (SELECT TRE.ID
                                                       FROM TASK TAS
                                                       JOIN TASK_RELATIONSHIP TAR ON ( TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'W')
                                                       JOIN TASK TRE              ON (TAR.TASK = TRE.ID)
                                                      WHERE TAS.ID IN ({$schedulesIdsString})
                                                        AND TRE.IS_ISSUE = 'N'
                                                        AND TAS.STATUS = 'A') ";
                }
        }


        $projectIdFilter = '';
        if(!empty($projectIds)) {
            $projectIdsAll = EntitiesUtil::getAllProjectLessIdIncludingHimByArray($projectIds);
            $projectIdFilter = "AND (" . EntitiesUtil::fixBugWhereINQuery($projectIdsAll, 'PRO.ID') . ")";
        }

        $meetingTaskStatusFilter = '';
        if(!empty($meetingTaskStatus)){
            $meetingTaskStatusFilter = "AND TAS.TASK_STATUS IN (" . implode(',', $meetingTaskStatus) . ")";
        }

        $betweenDateFilter = '';
        if (!empty($betweenDateTask)) {
            $betweenDateFilter = "AND TAS.CREATED_AT BETWEEN TO_DATE( '$betweenDateTask[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$betweenDateTask[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
        }

        $taskLabelsFilter = '';
        if (!empty($taskLabels)) {
            $taskLabelsFilter = "AND TLB.LABEL IN  (" . implode(',', $taskLabels) . ")";
        }
        return array($projectIdFilter, $meetingTaskStatusFilter, $betweenDateFilter, $taskLabelsFilter, $taskScheduleFilter);
    }

    public function findIssueMeetingPlannig($projectId, $param, $page, $pageSize)
    {
        try{
            $limit = $page * $pageSize;
            $offset = ($page - 1) * $pageSize;

            $project = $this->projectRepository->find($projectId);
            $isOccurrences = !!$project->getParent();
            $meeting = $this->getMeetingByProject($project);
            $meetingProjects = $this->meetingProjectRepository->findBy(array('meeting' => $meeting));

            $isAllowScheduleFilter = $meeting->getAllowScheduleFilter();
            list($projectIdFilter, $meetingTaskStatusFilter, $betweenDateFilter, $issueLabelsFilter, $issueScheduleFilter) = $this->createFiltersFindIssuesMeetingPlannig($meetingProjects, $meeting, $projectId, $isAllowScheduleFilter);
            $scheduleFilter = $projectIdFilter . $meetingTaskStatusFilter . $betweenDateFilter . $issueLabelsFilter;

            if(empty($betweenDateFilter) && empty($meetingTaskStatusFilter) && $isAllowScheduleFilter == 'Y'){
                $scheduleFilter = '';
                $issueScheduleFilter = str_replace("OR", "AND" , $issueScheduleFilter);
            } else if(empty($betweenDateFilter) && empty($meetingTaskStatusFilter) && $isAllowScheduleFilter == 'N'){
                return [];
            }else if ((!empty($betweenDateFilter) || !empty($meetingTaskStatusFilter)) && $isAllowScheduleFilter == 'N'){
                $issueScheduleFilter = '';
            }

            $filter = EntitiesUtil::createQueryFiltered($param);
            return $this->taskRepository->findIssueMeetingPlannig($scheduleFilter, $issueScheduleFilter, $filter, $limit, $offset);
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    public function createFiltersFindIssuesMeetingPlannig(array $meetingProjects,Entities\Meeting $meeting, $projectId, $isAllowScheduleFilter)
    {
        $projectIds = $this->getProjectIdsIssue($meetingProjects);
        $betweenDateIssue = $this->getBetweenDateIssueFilter($meeting);
        $meetingTaskStatusIssue = $this->getMeetingTaskStatusIssueFilter($meeting);
        $resourceProjectSponsors = $this->getResourceProjectSponsorsFilter($projectId);
        $issueLabels = $this->getIssueLabelsFilter($meeting);

        $issueScheduleFilter = '';
        if($isAllowScheduleFilter = 'Y'){
            $schedulesIds = $this->taskRepository->findSchedulesIdsByMeeting($projectId);
            if(!empty($schedulesIds)){
                $schedulesIds = EntitiesUtil::getArrayFromArrayAssociative($schedulesIds);
                $schedulesIdsString = implode(',', $schedulesIds);
                $issueScheduleFilter = "OR TAS.ID IN (SELECT TRE.ID
                                                        FROM TASK TAS
                                                        JOIN TASK_RELATIONSHIP TAR ON ( TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'W')
                                                        JOIN TASK TRE              ON (TAR.TASK = TRE.ID)
                                                        LEFT JOIN TASK_STATUS TST       ON TST.ID = TRE.TASK_STATUS
                                                        WHERE TRE.IS_ISSUE = 'Y'
                                                          AND TAS.ID IN ({$schedulesIdsString})
                                                          AND TAS.STATUS = 'A') ";
            }
        }


        $projectIdFilter = '';
        if(!empty($projectIds)) {
            $projectIdsAll = EntitiesUtil::getAllProjectLessIdIncludingHimByArray($projectIds);
            $projectIdFilter = "AND (" . EntitiesUtil::fixBugWhereINQuery($projectIdsAll, 'PRO.ID') . " )";
        }

        $meetingTaskStatusFilter = '';
        if(!empty($meetingTaskStatusIssue)){
            $meetingTaskStatusFilter = "AND TAS.TASK_STATUS IN (" . implode(',', $meetingTaskStatusIssue) . ") ";
        }

        $betweenDateFilter = '';
        if (!empty($betweenDateIssue)) {
            $betweenDateFilter = "AND (TAS.CREATED_AT BETWEEN TO_DATE( '$betweenDateIssue[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$betweenDateIssue[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')) ";
        }

        $issueLabelsFilter = '';
        if (!empty($issueLabels)) {
            $issueLabelsFilter = "AND TLB.LABEL IN (" . implode(',', $issueLabels) . ") ";
        }
        return array($projectIdFilter, $meetingTaskStatusFilter, $betweenDateFilter, $issueLabelsFilter, $issueScheduleFilter);
    }

}
