<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Zeedhi\DevOps\Util\Utilities as DevopsUtilities;


class ProductFiles extends \Doctrine\ORM\EntityRepository {
    /**
     * @param $productId
     * @param $subreleaseId
     * @param $fileName
     * @param $relativePath
     * @return array
     */
    public function findProductFilesByFileNameAndRelativePath($productId, $subreleaseId, $fileName, $relativePath) {
        $query = <<<SQL
			SELECT SP.ID, PF.FILE_NAME, PF.RELATIVE_PATH, PF.TYPE
              FROM PRODUCT_FILES PF
              LEFT JOIN SUBRELEASE_PRODUCT_FILES SP ON SP.PRODUCT_FILES_ID = PF.ID AND SP.SUBRELEASE_ID = :SUBRELEASE
             WHERE PF.PRODUCT_ID = :PRODUCT
               AND PF.FILE_NAME = :FILE_NAME
               AND PF.RELATIVE_PATH = :RELATIVE_PATH
               AND SP.ID IS NOT NULL
SQL;
        $params = array( 'PRODUCT'       => $productId,
                         'SUBRELEASE'    => $subreleaseId,
                         'FILE_NAME'     => $fileName,
                         'RELATIVE_PATH' => $relativePath );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductFilesToExecuteByVersion($productId, $version, $revision) {
        $query = <<<SQL
			SELECT NAME, FILE_NAME, RELATIVE_PATH, TYPE,
                   DECODE(VERSION, NULL, '0.0', VERSION || '.' || REVISION) AS VERSION
              FROM (SELECT PF.NAME, PF.FILE_NAME, PF.RELATIVE_PATH, PF.TYPE,
                           MIN(PV.VERSION) AS VERSION, MIN(PV.REVISION) AS REVISION
                      FROM PRODUCT_FILES PF
                      LEFT JOIN PRODUCT_VERSION PV ON PV.ID = PF.PRODUCT_VERSION_ID
                     WHERE PF.PRODUCT_ID = :PRODUCT
                       AND (TO_NUMBER( LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0')) >= TO_NUMBER(LPAD(:VERSION, 5, '0') || LPAD(:REVISION, 5, '0')) 
                            OR PF.PRODUCT_VERSION_ID IS NULL)
                  GROUP BY PF.NAME, PF.FILE_NAME, PF.RELATIVE_PATH, PF.TYPE)
          ORDER BY NAME
SQL;
        $params = array( 'PRODUCT'  => $productId,
                         'VERSION'  => $version,
                         'REVISION' => $revision );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @param $fileName
     * @param $relativePath
     * @param $id
     * @return array
     */
    public function findProductFilesByProductAndFileNameAndRelativePath(
                        $productId, $fileName, $relativePath, $id, $organizationId, $limit = null, $offset = null
                    )
    {
        $query = <<<SQL
			SELECT PF.ID, PF.NAME, PF.FILE_NAME, PF.RELATIVE_PATH, PF.TYPE, PF.IS_REQUIRED
              FROM PRODUCT_FILES PF
             WHERE PF.PRODUCT_ID = :PRODUCT
               AND PF.ORGANIZATION_ID = :ORGANIZATION
               AND PF.FILE_NAME = :FILE_NAME
               AND PF.RELATIVE_PATH = :RELATIVE_PATH
               AND PF.ID <> :ID
               AND PF.PARENT_ID IS NULL
               AND NOT EXISTS (SELECT PL.ID
                                 FROM PRODUCT_FILES PL
                                WHERE PL.PARENT_ID = PF.ID)
SQL;
        $params = array( 'PRODUCT'       => $productId,
                         'ID'            => $id,
                         'FILE_NAME'     => $fileName,
                         'ORGANIZATION'  => $organizationId,
                         'RELATIVE_PATH' => $relativePath
        );
        $query = DevopsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @param $version
     * @param $revision
     * @param $filter
     * @return array
     */
    public function findVersionProductFiles($productId, $version, $revision, $filter = '')
    {
        $query = <<<SQL
        SELECT *
          FROM (SELECT PF.ID, PF.PRODUCT_ID, PF.NAME, PF.FILE_NAME, PF.RELATIVE_PATH, PF.TYPE,
                       PF.IS_REQUIRED, PF.TEXT, PF.STATE, PF.IS_WINDOWS, PF.PRODUCT_VERSION_ID,
                       DECODE(PV.VERSION, NULL, '0.0', (PV.VERSION || '.' || PV.REVISION)) AS PRODUCT_VERSION
                  FROM PRODUCT_FILES PF
             LEFT JOIN PRODUCT_VERSION PV ON PV.ID = PF.PRODUCT_VERSION_ID
            INNER JOIN (SELECT PF.FILE_NAME, PF.RELATIVE_PATH,
                               NVL(MIN(PV.VERSION), '0') AS VERSION, NVL(MIN(PV.REVISION), '0') AS REVISION
                          FROM PRODUCT_FILES PF
                     LEFT JOIN PRODUCT_VERSION PV ON PV.ID = PF.PRODUCT_VERSION_ID
                         WHERE PF.PRODUCT_ID = :PRODUCT
                           AND (TO_NUMBER( LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0')) >= TO_NUMBER(LPAD(:VERSION, 5, '0') || LPAD(:REVISION, 5, '0')) 
                                 OR PF.PRODUCT_VERSION_ID IS NULL)
                      GROUP BY PF.FILE_NAME, PF.RELATIVE_PATH
                       ) VR ON VR.VERSION       = NVL(PV.VERSION, '0')
                           AND VR.REVISION      = NVL(PV.REVISION, '0')
                           AND VR.FILE_NAME     = PF.FILE_NAME
                           AND VR.RELATIVE_PATH = PF.RELATIVE_PATH
                 WHERE PF.PRODUCT_ID = :PRODUCT
              ORDER BY NAME, PARENT_ID ASC NULLS FIRST, PV.VERSION DESC, PV.REVISION DESC)
         WHERE 1 = 1
               $filter
SQL;
        $params = array(
            'PRODUCT'  => $productId,
            'VERSION'  => $version,
            'REVISION' => $revision,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}